#!/usr/bin/perl
#####
#
# Copyright (C) 2002 Laurent Oudot <oudot.laurent@wanadoo.fr>
# All Rights Reserved
#
# This file is part of the Prelude program.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
#
#####

##
#
# Latest release of this perl proof of concept can be found on
# http://www.rstack.org/oudot/prelude/
#
# More fun on http://www.rstack.org  [official web site of the rstack team]
# 
##

use DBI;
use Getopt::Std;

$::period=30;	#period of time used to parse similar alerts
$::alertid;	#specific alert id on which we want to work to find correlation
$usage="\n<== Small proof of concept of IDMEF correlation (made by oudot\@rstack.org) ==>\n\n==> $0 can be used to make specific correlation\n\n=> You can use it on one alert, like that :\n\n $0 -a <alertid> [-p period]\n\n <alertid> is the alert id on which you want to find related correlation.\n [period] is the period of time that will be used to check other alerts.\n (60 for 1 minute, 360 for one hour, 86400 for one day)\n\n=> You can also use it to parse the whole database :\n\n $0 -B [-p period]\n\n This method could be use in a cron like job...\n [period] is the period of time that will be used to check other alerts.\n\n=> Example of use :\n   you want to find alerts that are related on periods of one day.\n   This poor tool will try to help you at analysing your logs:\n   $0 -B -p 86400\n";

## Beginning
#
getopts('p:a:n:B')
	or die "$usage";
if( $opt_a )
{
	$::alertid=$opt_a;
}
elsif( ! $opt_B )
{
	die "$usage";
}
if( $opt_p )
{
	$::period=$opt_p;
}

## Connection to the database
#  Please change with your own real values (login, password...)
$db = DBI->connect("DBI:mysql:database=prelude:host=localhost","prelude","prelude");


## Wonna try the whole database ?
#
if( $opt_B )
{
	$reqB= $db->prepare("select ident FROM Prelude_Alert");
	$reqB->execute() or die "problem on select";
	while($rowB=$reqB->fetchrow_arrayref)
	{
		correlation($rowB->[0]);
	}
	$reqB->finish;
}
## Wonna try with one alertid only
#
else
{
	correlation($::alertid);
}


## Function for correlation
#
sub correlation {
if( $_[0] )
{
  $::alertid=$_[0];
}

print  "#################################################\n# Proof of concept for a small correlation agent\n# using source address to work\n# Contact: oudot\@rstack.org\n#################################################\n\n";
print "\n# Source address correlation for alert $::alertid\n# with $::period of seconds used as the period to analyse\n";

## Search if the Alert is ever in a correlated alert
#  We could do a correlation again.. but it's just a first proof of concept
$reqs = $db->prepare("select ident, alert_ident FROM Prelude_CorrelationAlert_Alerts WHERE alert_ident = $::alertid");
$reqs->execute() or die "problem on select";
while($row=$reqs->fetchrow_arrayref)
{
	print "\n# Alert $row->[1] is ever correlated in the correlation id: $row->[0]\n\n";
	return;
}
$reqs->finish;



## Search information about the alert analyzed
#
$reqs = $db->prepare("select t.alert_ident, t.time, a.address FROM Prelude_Address a, Prelude_DetectTime t WHERE a.alert_ident = $::alertid AND a.parent_type = 'S' AND t.alert_ident = $::alertid");
$reqs->execute() or die "problem on select";
while($row=$reqs->fetchrow_arrayref)
{
	print "# Alert $row->[0] occured on $row->[1] from $row->[2]\n\n";
}
$reqs->finish;

## Get the maximum CorrelationAlert.ident
#  There is no mutex on this colon, so it supposes you don't write with
#  another program in the CorrelationAlert
$reqs = $db->prepare("select max(ident) FROM Prelude_CorrelationAlert;");
$reqs->execute() or die "problem on select";
while($row=$reqs->fetchrow_arrayref)
{
	$corident=$row->[0]+1;
	print "# New correlationAlert number usable : $corident\n";
}
$reqs->finish;

## Search of the possible correlated alerts
#
$reqs = $db->prepare("select m.alert_ident, m.time, M.address from Prelude_DetectTime m, Prelude_DetectTime n, Prelude_Address M, Prelude_Address N WHERE n.time < m.time AND m.time < (n.time + INTERVAL $::period SECOND) AND n.alert_ident = $::alertid AND N.alert_ident = n.alert_ident AND N.parent_type = 'S' AND M.parent_type = 'S' AND M.address = N.address AND M.alert_ident = m.alert_ident;");
$reqs->execute() or die "problem on select";

print "# List of possible correlated alerts with $::period as a basis of analyse\n# Alertid\tDate\t\t\tAddress\n";

## Debug purpose only
#
if($opt_n)
{
 $corident=$opt_n;
}

## As there are correlated alerts, we must create a specific id
#  for the CorrelationAlert table 
if($reqs->rows != 0)
{
	$reqi = $db->prepare("insert into Prelude_CorrelationAlert (name,ident) VALUES ('Source correlation (period=$::period sec) by prelude-correlation-agent',$corident);");
	$reqi->execute() or die "problem on insert";
	$reqi->finish;
}
else
{
	return;
}

while($row=$reqs->fetchrow_arrayref)
{
	print "  $row->[0]\t\t$row->[1]\t$row->[2]\n";

	$reqi = $db->prepare("insert into Prelude_CorrelationAlert_Alerts (ident,alert_ident) VALUES ($corident,$row->[0]);");
	$reqi->execute() or die "problem on insert";
	$reqi->finish;
}

## we must not forget the alert itself...
$reqi = $db->prepare("insert into Prelude_CorrelationAlert_Alerts (ident,alert_ident) VALUES ($corident, $::alertid);");
$reqi->execute() or die "problem on insert";
$reqi->finish;


$reqs->finish;

print "##\n\n";
}

