#!/usr/bin/perl
#####
#
# Copyright (C) 2002 Laurent Oudot <oudot.laurent@wanadoo.fr>
# All Rights Reserved
#
# This file is part of the Prelude program.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
#
#####

##
#
# Latest release of this perl proof of concept can be found on
# http://www.rstack.org/oudot/prelude/
#
# More fun on http://www.rstack.org  [official web site of the rstack team]
# 
##

use DBI;
use Getopt::Long;

$count_treated=0;
$count_found=0;

my $db_type="";
my $db_name="";
my $db_host="";
my $db_user="";
my $db_password="";

my $file_config="";

my $verbose=0;
my $interactive=0; #not yet implemented
my $print_help=0;
my $print_version=0;

my $mode="one"; # big || one

my $tiny_version = "v1.0";
my $version="\n
This is the Prelude Correlation Vulnerability Finder, $tiny_version 

Copyright 2002, Laurent Oudot <oudot.laurent\@wanadoo.fr>
\n";

my $alertid;	#specific alert id on which we want to work to find correlation

my $usage="\nPrelude Correlation Vulnerability Finder, $tiny_version
Copyright 2002, Laurent Oudot

This tool can find correlation between the alerts stocked in your database, and the vulnerability reports that have been imported before in this database owing to the other associated tool. Those vulnerability reports must have been created owing to assessment tools (audits).  
Audit tools supported : Nessus (http://www.nessus.org)

$0 	[-B]   [--BIG]
	or [-a alert_number]   [--alertid alert_number]	
	-c <vuln.conf>   --config <vuln.conf>
	[-v] [--verbose]
	[-h] [--help]
	[--version]
Use -B to parse the whole database, and -a to look at one specific alert.
Example: $0 -B -c /usr/local/etc/prelude-correlation/vuln.conf

The vuln.conf should just contain one line like this \"dbType:dbName:dbHost:dbUser:dbPassword:\"
Example of vuln.conf file content:   mysql:prelude:localhost:prelude:guessme:
\n";

## Main
#
args();
conf();
db();
parse();
result();

sub args
{
	unless (GetOptions(
	"B|BIG"		=> sub { shift; $mode="big" },
	"a|alertid=s"	=> \$alertid,
	"c|config=s"	=> \$file_config,
	"h|help"	=> \$print_help,
	"version"	=> \$print_version,
	"v|verbose"	=> \$verbose,
	))
	{
		die "$usage";
	};
	if( $print_help )
	{
		die "$usage";
	}
	if( $prinf_version )
	{
		die "$version";
	}
	if( ! $file_config )
	{
		die "$usage";
	}
}

## Read the configuration
#
sub conf
{
	open(CONF, $file_config) || die "Can't open the configuration file: $file_config: $!\n";
	while( <CONF> )
	{
		($db_type,$db_name,$db_host,$db_user,$db_password) = split(/\:/);
	}
	close(CONF);
}

## Connection to the database
#
sub db
{
	print "*** Trying to connect to the database\n type:$db_type\t database:$db_name\n hostname:$db_host\t account:$db_user\n";
	if($verbose)
	{
		printf " password:$db_password\n";
	}
	$db = DBI->connect("DBI:$db_type:database=$db_name:host=$db_host","$db_user","$db_password") || die "Cannot connect to the database: $!\n";
}


## Parse the database to match the alerts on which we gonna work
#
sub parse
{
	if( $mode eq "big" )
	{
		print "\n*** Parsing the database to get the alerts id\n";
		$reqB= $db->prepare("select ident FROM Prelude_Alert ORDER BY ident");
		$reqB->execute() or die "Problem on select ident";
		print "\n*** Trying to search for correlation\n";
		while($rowB=$reqB->fetchrow_arrayref)
		{
			correlation_vulnerability($rowB->[0]);
			$count_treated++;
		}
		$reqB->finish;
	}
	else
	{
		print "\n*** One alert to look at\n";
		print "\n*** Trying to search for correlation\n";
		correlation_vulnerability($alertid);
	}
}

## Search correlation through vulnerability reports that have been inserted
#  in the prelude database
#
sub correlation_vulnerability
{
	my $alertid;
	if( $_[0] )
	{
		$alertid=$_[0];
	}

	$reqs = $db->prepare("select a.address, s.port, s.protocol FROM Prelude_Address a, Prelude_Service s WHERE a.alert_ident = $alertid AND s.alert_ident = $alertid");
	$reqs->execute() or die "problem on select";
	while($row=$reqs->fetchrow_arrayref)
	{
		$address=$row->[0];
		$port=$row->[1];
		$protocol=$row->[2];
	}
	$reqs->finish;

	$reqc = $db->prepare("select script_id,type,data,date FROM Prelude_CorrelationVulnerability 
	WHERE address = '$address' 
	AND port='$port'
	AND protocol = '$protocol'");
	$reqc->execute() or die "problem on select";
	while($rowc=$reqc->fetchrow_arrayref)
	{
		$script_id=$rowc->[0];
		$type=$rowc->[1];
		$data=$rowc->[2];
		$date=$rowc->[3];
		$data=~ s/;/\n/g;
		print "*** Vulnerability Correlation ***\n* Level: $type * Audit report inserted on $date* Alert $alertid for $address:$port ($protocol) (script id=$script_id) : \n$data\n***\n\n";
		$count_found++;
	}
}

## Print some results at the end of the process
#
sub result
{
	print "*** Operation of vulnerability correlation search finnished\n\n";
	print " Number of alerts treated in the database : $count_treated\n";
	print " Number of correlations of vulnerability  : $count_found\n";
}

