#!/usr/bin/perl
#####
#
# Copyright (C) 2002 Laurent Oudot <oudot.laurent@wanadoo.fr>
# All Rights Reserved
#
# This file is part of the Prelude program.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
#
#####

##
#
# Latest release of this perl proof of concept can be found on
# http://www.rstack.org/oudot/prelude/
#
# More fun on http://www.rstack.org  [official web site of the rstack team]
# 
##

use Getopt::Long;
use DBI;
use Time::localtime;


my $count_add=0;
my $count_del=0;

my $db_type="";
my $db_name="";
my $db_host="";
my $db_user="";
my $db_password="";

my $file_config="";
my $file_report="";

my $verbose=0;
my $interactive=0; #not yet implemented
my $print_help=0;
my $print_version=0;

my $tiny_version = "v1.0";
my $version="\n
This is the Prelude Correlation Vulnerability Importer, $tiny_version 

Copyright 2002, Laurent Oudot <oudot.laurent\@wanadoo.fr>
\n";

my $usage="\nPrelude Correlation Vulnerability Importer, $tiny_version
Copyright 2002, Laurent Oudot

This tool can import reports of vulnerabilities audits made by other tools. Those reports will then be integrated in the database used by a prelude-manager to stock alerts of security. Owing to this import, frontends for Prelude-IDS will be able to add correlation about vulnerability found before.
Audit tools supported : Nessus (http://www.nessus.org)

$0 
	-i <nessus_report.nsr>   --import <nessus_report.nsr>
	-c <vuln.conf>   --config <vuln.conf>
	[-v] [--verbose]
	[-h] [--help]
	[--version]
Example: $0 -i /mnt/net/audits/reports/192.168.1.254.nsr -c /usr/local/etc/prelude-correlation/vuln.conf

The vuln.conf should just contain one line like this \"dbType:dbName:dbHost:dbUser:dbPassword:\"
Example of vuln.conf file content:   mysql:prelude:localhost:prelude:guessme:
\n";

my @hosts = ();

## Main
#
args();
conf();
db();
parse();
purge(@hosts);
add(@hosts);
result();


## Initialization
#
sub args
{
	unless (GetOptions (
		"i|import=s"	=> \$file_report,
		"v|verbose"	=> \$verbose,
		"c|config=s"	=> \$file_config,
		"h|help"	=> \$print_help,
		"version"	=> \$print_version
		)) 
	{ 
			die "Problem with arguments ($!)\n"; 
	};
	if( $print_help )
	{
		die "$usage";
	}
	if( $print_version )
	{
		die "$version";
	}
	if( ! $file_report )
	{
		die "$usage";
	}
	$date = sprintf("%d/%s %s %d:%d\n", localtime->year() + 1900,localtime->mon(),localtime->mday(),localtime->hour(), localtime->min());
}

## Read the configuration
#
sub conf
{
	open(CONF, $file_config) || die "Can't open the configuration file: $file_config: $!\n";
	while( <CONF> )
	{
		($db_type,$db_name,$db_host,$db_user,$db_password) = split(/\:/);
	}
	close(CONF);
}

## Connection to the database
#
sub db
{
	print "***\nTrying to connect to the database\n type:$db_type\t database:$db_name\n hostname:$db_host\t account:$db_user\n";
	if($verbose)
	{
		printf " password:$db_password\n";
	}
	$db = DBI->connect("DBI:$db_type:database=$db_name:host=$db_host","$db_user","$db_password") || die "Cannot connect to the database: $!\n";
}



## Function to parse an audit report
#
sub parse
{
	$nsr = $file_report;
	print "***\nReading the report file: $nsr\n";
	open(NSR, $nsr) || die "Can't open $nsr: $!\n";
	while( <NSR> )
	{
		local($tmp);
		($address, $tmp, $script_id, $type, $data) = split(/\|/);
		if($type ne "NOTE" && $type ne "INFO" && $type ne "REPORT")
		{
			$script_id=0;
			$type="INFO";
			$data="Nessus reported that the port is opened ";
		}
		($port,$protocol) = split(/\//,$tmp);
		if($port eq "general")
		{
			$port=0;
		}
		else
		{
			($tmp,$port) = split(/\(/,$port);
			$protocol =~ s/\)//g;
		}
		chop $data;
		$data =~ s/\'/\\\'/g;
		push @hosts,($address,$port,$protocol,$script_id,$type,$data);
		if($verbose)
		{
			print ">> Reading : $address | $port | $protocol | $script_id | $type | $data \n";
		}
	}
	close(NSR);
	printf "Number of reports to deal with: %d\n", (scalar @hosts) / 6;
}

## Purge of the existing audits in the database
#
sub purge 
{
	my $i;
	my $max = (scalar @_) ;
	print "***\nDeleting existing related reports\n";
	for($i=0; $i < $max ; $i=$i+6)
	{
		$address = $_[$i];
		$req = $db->prepare("delete from Prelude_CorrelationVulnerability WHERE address='$address';");
		$req->execute() or die "Problem during the deletion process\n";
		$req->finish;
		$count_del++;
		if($verbose)
		{
			print ">> Deleting reports for $address\n";
		}
		else
		{
			print ".";
		}
	}
	print "\nEnd of the deletion process\n";
}

## Add the new elements of audits in the database
#
sub add
{	
	my $i;
	my $max = (scalar @_) ;
	printf "***\nAdding %d new reports\n", (scalar @_) / 6;
	for($i=0; $i < $max ; $i=$i+6)
	{
		$address   = $_[$i];
		$port      = $_[$i+1];
		$protocol  = $_[$i+2];
		$script_id = $_[$i+3];
		$type      = $_[$i+4];
		$data      = $_[$i+5];
		
		$req = $db->prepare("insert into Prelude_CorrelationVulnerability (address,port,protocol,script_id,type,data,date) VALUES ('$address','$port','$protocol','$script_id','$type','$data','$date');");
		$req->execute() or die "problem on insert";
		$req->finish;
		if($verbose)
		{
			print ">> Adding : $address | $port | $protocol | $script_id | $type | $data \n";
		}
		$count_add++;
	}
	print "End of the integration of the reports\n";
}

## Print some results at the end of the process
#
sub result
{
	print "***\nOperation of import succeeded\n\n";
	print " Number of records deleted in the database : $count_del\n";
	print " Number of records written in the database : $count_add\n";
	print " Date used in the records                  : $date\n";
}
